from __future__ import annotations

import re
import typing as t
from sqlglot import exp, generator
from sqlglot.dialects.dialect import (
    approx_count_distinct_sql,
    count_if_to_sum,
    rename_func,
    time_format,
    Dialect,
)
from sqlglot.helper import seq_get
from sqlglot.dialects.mysql import MySQL
from sqlglot.parser import binary_range_parser
from sqlglot.tokens import TokenType

if t.TYPE_CHECKING:
    from sqlglot._typing import E

DATE_DELTA_INTERVAL = {
    "year": "year",
    "yyyy": "year",
    "yy": "year",
    "quarter": "quarter",
    "qq": "quarter",
    "q": "quarter",
    "month": "month",
    "mm": "month",
    "m": "month",
    "week": "week",
    "ww": "week",
    "wk": "week",
    "day": "day",
    "dd": "day",
    "d": "day",
}


def no_paren_current_date_sql(self, expression: exp.CurrentDate) -> str:
    zone = self.sql(expression, "this")
    return f"CURRENT_DATE() AT TIME ZONE {zone}" if zone else "CURRENT_DATE()"


def handle_array_concat(self, expression: exp.ArrayStringConcat) -> str:
    this = self.sql(expression, "this")
    expr = self.sql(expression, "expressions")
    if expr == "":
        return f"CONCAT_WS('',{this})"
    return f"CONCAT_WS({expr}, {this})"


def handle_array_to_string(self, expression: exp.ArrayToString) -> str:
    this = self.sql(expression, "this")
    sep = self.sql(expression, "sep")
    null_replace = self.sql(expression, "null_replace")
    result = f"ARRAY_JOIN({this},{sep}"
    if null_replace:
        result += f",{null_replace}"

    result += ")"

    return result


def handle_concat_ws(self, expression: exp.ConcatWs) -> str:
    delim, *rest_args = expression.expressions
    rest_args_sql = ", ".join(self.sql(arg) for arg in rest_args)
    return f"CONCAT_WS({self.sql(delim)}, {rest_args_sql})"


def handle_date_diff(self, expression: exp.DateDiff) -> str:
    unit = self.sql(expression, "unit").lower()
    expressions = self.sql(expression, "expression")
    this = self.sql(expression, "this")
    if unit == "microsecond":
        return f"MICROSECONDS_DIFF({this}, {expressions})"
    elif unit == "millisecond":
        return f"MILLISECONDS_DIFF({this}, {expressions})"
    elif unit == "second":
        return f"SECONDS_DIFF({this}, {expressions})"
    elif unit == "minute":
        return f"MINUTES_DIFF({this}, {expressions})"
    elif unit == "hour":
        return f"HOURS_DIFF({this}, {expressions})"
    elif unit == "day":
        return f"DATEDIFF({this}, {expressions})"
    elif unit == "month":
        return f"MONTHS_DIFF({this}, {expressions})"
    elif unit == "year":
        return f"YEARS_DIFF({this}, {expressions})"
    return f"DATEDIFF({this}, {expressions})"


def handle_date_trunc(self, expression: exp.DateTrunc) -> str:
    unit = self.sql(expression, "unit").strip("\"'").lower()
    this = self.sql(expression, "this")
    if unit.isalpha():
        mapped_unit = DATE_DELTA_INTERVAL.get(unit) or unit
        return f"DATE_TRUNC({this}, '{mapped_unit}')"
    if unit.isdigit() or unit.lstrip("-").isdigit() or this.isdigit():
        if this.isdigit():
            return f"TRUNCATE({this})"
        return f"TRUNCATE({this}, {unit})"
    return f"DATE({this})"


def handle_geography(
    self, expression: exp.StAstext
) -> str:  # Realize the identification of geography
    this = self.sql(expression, "this").upper()
    match = re.search(r"POINT\(([-\d.]+) ([-\d.]+)\)", this)
    if match is None:
        return f"ST_ASTEXT(ST_GEOMETRYFROMWKB({this}))"
    x = float(match.group(1))
    y = float(match.group(2))
    return f"ST_ASTEXT(ST_POINT{x, y})"


def handle_log(self, expression: exp.Log) -> str:
    this = self.sql(expression, "this")
    expression = self.sql(expression, "expression")

    if expression == "":
        return self.func("LOG10", this)
    return self.func("LOG", this, expression)


def handle_filter(self, expr: exp.Filter) -> str:
    expression = expr.copy()
    self.sql(expr, "this")
    expr = expression.expression.args["this"]
    agg = expression.this.key
    spec = expression.this.args["this"]
    case = (
        exp.Case()
        .when(
            expr,
            spec,
        )
        .else_("0")
    )
    return f"{agg}({self.sql(case)})"


def _string_agg_sql(self: Doris.Generator, expression: exp.GroupConcat) -> str:
    expression = expression.copy()
    separator = expression.args.get("separator") or exp.Literal.string(",")

    order = ""
    this = expression.this
    if isinstance(this, exp.Order):
        if this.this:
            this = this.this.pop()
        order = self.sql(expression.this)  # Order has a leading space
    if isinstance(separator, exp.Chr):
        separator = "\n"
        return f"GROUP_CONCAT({self.format_args(this)}{order},'{separator}')"
    return f"GROUP_CONCAT({self.format_args(this, separator)}{order})"


def handle_range(self, expression: exp.GenerateSeries) -> str:
    start = self.sql(expression, "start")
    end = str(int(self.sql(expression, "end")) + 1)
    step = self.sql(expression, "step")

    if step == "":
        return self.func("Array_Range", start, end)

    return self.func("Array_Range", start, end, step)


def handle_regexp_extract(self, expr: exp.RegexpExtract) -> str:
    this = self.sql(expr, "this")
    expression = self.sql(expr, "expression")
    occurrence = self.sql(expr, "occurrence")
    if occurrence == "":
        return f"REGEXP_EXTRACT_ALL({this}, '({expression[1:-1]})')"

    return f"REGEXP_EXTRACT({this}, '({expression[1:-1]})', {occurrence})"


def build_time_format(time_format: str | None) -> str | None:
    if time_format is not None:
        # Replace "mi" with "%i"
        time_format = time_format.replace("mi", "%i")
        # postgres %M->%i
        time_format = time_format.replace("%M", "%i")
        # Replace both "hh24" and "HH24" with "%H", case-insensitively
        time_format = re.sub("hh24", "%H", time_format, flags=re.IGNORECASE)
        # Replace both "sYYYY" and "YYYY" with "%Y", case-insensitively
        time_format = re.sub(r"s{0,1}%y", "%Y", time_format, flags=re.IGNORECASE)
        # Remove all digits from the time_format
        time_format = re.sub(r"\d", "", time_format)

    return time_format


def handle_to_date(
    self: Doris.Generator,
    expression: exp.TsOrDsToDate | exp.StrToTime | exp.ToChar | exp.TimeToStr | exp.StrToDate,
) -> str:
    this = self.sql(expression, "this")
    time_format = self.format_time(expression)
    time_format = build_time_format(time_format)

    if time_format and time_format not in (Doris.TIME_FORMAT, Doris.DATE_FORMAT):
        if isinstance(expression, exp.StrToDate):
            return f"STR_TO_DATE({this}, {time_format})"
        return f"DATE_FORMAT({this}, {time_format})"
    if isinstance(expression.this, exp.TsOrDsToDate):
        return this
    return f"TO_DATE({this})"


def handle_to_char(self: Doris.Generator, expression: exp.ToChar | exp.TimeToStr) -> str:
    this = self.sql(expression, "this")
    time_format = self.format_time(expression)
    time_format = build_time_format(time_format)
    if isinstance(expression.this, exp.Interval):
        raise ValueError(
            "eg. Postgres:to_char(interval '15h 2m 12s', 'HH24:MI:SS') → 15:02:12 not support transform"
        )
    if time_format and time_format not in (Doris.TIME_FORMAT, Doris.DATE_FORMAT):
        return f"DATE_FORMAT({this}, {time_format})"
    if isinstance(expression.this, exp.TsOrDsToDate):
        return this
    return f"CAST({this} AS STRING)"


def handle_replace(self, expression: exp.Replace) -> str:
    this = self.sql(expression, "this")
    old = self.sql(expression, "old")
    new = self.sql(expression, "new")
    if new == "":
        return f"REPLACE({this},{old},'')"
    return f"REPLACE({this},{old},{new})"


def handle_rand(self, expr: exp.Rand) -> str:
    min = self.sql(expr, "this")
    max = self.sql(expr, "expression")
    if min == "" and max == "":
        return "RANDOM()"
    elif max == "":
        return f"FLOOR(RANDOM()*{min}.0)"
    else:
        temp = int(max) - int(min)
        return f"FLOOR(RANDOM()*{temp}.0+{min}.0)"


def _str_to_unix_sql(self: generator.Generator, expression: exp.StrToUnix) -> str:
    return self.func("UNIX_TIMESTAMP", expression.this, time_format("doris")(self, expression))


# Matches a numeric or string value in a string
def extract_value_from_string(expression):
    match = re.search(r"(\b\d+\b|\b\w+(?:\.\w+)*\b)", expression)
    if match:
        return match.group(0)
    else:
        return None


def arrow_json_extract_sql(self, expression: exp.JSONExtract) -> str:
    expr = self.sql(expression, "expression").strip("\"'")
    expr = extract_value_from_string(expr)
    if expr.isdigit():
        return f"JSONB_EXTRACT({self.sql(expression, 'this')},'$[{expr}]')"
    return f"JSONB_EXTRACT({self.sql(expression, 'this')},'$.{expr}')"


def arrow_jsonb_extract_sql(self, expression: exp.JSONBExtract) -> str:
    expr = self.sql(expression, "expression").strip("\"'")
    values = [key.strip() for key in expr[1:-1].split(",")]
    json_path = []
    for value in values:
        if value.isdigit():
            json_path.append(f"[{value}]")
        else:
            json_path.append(value)
    path = ".".join(json_path)
    return f"JSONB_EXTRACT({self.sql(expression, 'this')},'$.{path}')"


def arrow_json_extract_scalar_sql(self, expression: exp.JSONExtractScalar) -> str:
    # Extract the numerical values or strings among them
    expr = self.sql(expression, "expression").strip("\"'")
    expr = extract_value_from_string(expr)
    if expr.isdigit():
        return f"JSON_EXTRACT({self.sql(expression, 'this')},'$.[{expr}]')"
    return f"JSON_EXTRACT({self.sql(expression, 'this')},'$.{expr}')"


def arrow_jsonb_extract_scalar_sql(self, expression: exp.JSONBExtractScalar) -> str:
    expr = self.sql(expression, "expression").strip("\"'")
    values = [key.strip() for key in expr[1:-1].split(",")]
    json_path = []
    for value in values:
        if value.isdigit():
            json_path.append(f"[{value}]")
        else:
            json_path.append(value)
    path = ".".join(json_path)
    return f"JSON_EXTRACT({self.sql(expression, 'this')},'$.{path}')"


def _json_extract_sql(
    self: Doris.Generator,
    expression: exp.JSONExtract | exp.JSONExtractScalar | exp.JSONBExtract | exp.JSONBExtractScalar,
) -> str:
    return self.func(
        "JSONB_EXTRACT",
        expression.this,
        expression.expression,
        # *json_path_segments(self, expression.expression),
        expression.args.get("null_if_invalid"),
    )


def _trim_sql(self: Doris.Generator, expression: exp.Trim) -> str:
    target = self.sql(expression, "this")
    trim_type = self.sql(expression, "position")
    remove_chars = self.sql(expression, "expression")
    collation = self.sql(expression, "collation")

    # Use TRIM/LTRIM/RTRIM syntax if the expression isn't mysql-specific
    if not remove_chars and not collation:
        return self.trim_sql(expression)
    if trim_type == "LEADING":
        return self.func("LTRIM", target, remove_chars)
    elif trim_type == "TRAILING":
        return self.func("RTRIM", target, remove_chars)
    else:
        return self.func("TRIM", target, remove_chars)


def timestamptrunc_sql(self: Doris.Generator, expression: exp.TimestampTrunc) -> str:
    this = self.sql(expression, "this")
    unit = self.sql(expression, "unit").strip("\"'").lower()
    mapped_unit = DATE_DELTA_INTERVAL.get(unit) or unit
    return f"DATE_TRUNC({this}, '{mapped_unit}')"


def attimezone_sql(self: Doris.Generator, expression: exp.AtTimeZone) -> str:
    this = self.sql(expression, "this")
    timezone = self.sql(expression, "zone")
    return f"CONVERT_TZ({this},'UTC',{timezone})"


class Doris(MySQL):
    INDEX_OFFSET = 1
    DATE_FORMAT = "'yyyy-MM-dd'"
    DATEINT_FORMAT = "'yyyyMMdd'"
    TIME_FORMAT = "'yyyy-MM-dd HH:mm:ss'"
    NULL_ORDERING = "nulls_are_frist"
    DPIPE_IS_STRING_CONCAT = True
    TIME_MAPPING = {
        **MySQL.TIME_MAPPING,
        "%Y": "yyyy",
        "%m": "MM",
        "%d": "dd",
        "%s": "ss",
        "%H": "HH",
        "%i": "mm",
    }

    def quote_identifier(self, expression: E, identify: bool = True):
        # Add quote_identifier to doris keywords
        if (
            isinstance(expression, exp.Identifier)
            and not isinstance(expression.parent, exp.Func)
            and expression.name.upper() in Doris.KeyWords
        ):
            return Dialect.get_or_raise(Dialect).quote_identifier(
                expression=expression, identify=identify
            )

        return expression

    class Parser(MySQL.Parser):
        RANGE_PARSERS = {
            **MySQL.Parser.RANGE_PARSERS,
            TokenType.MATCH_ANY: binary_range_parser(exp.MatchAny),
            TokenType.MATCH_ALL: binary_range_parser(exp.MatchAll),
            TokenType.MATCH_PHRASE: binary_range_parser(exp.MatchPhrase),
        }
        FUNCTIONS = {
            **MySQL.Parser.FUNCTIONS,
            "ARRAY_SHUFFLE": exp.Shuffle.from_arg_list,
            "ARRAY_RANGE": exp.Range.from_arg_list,
            "ARRAY_SORT": exp.SortArray.from_arg_list,
            "COLLECT_LIST": exp.ArrayAgg.from_arg_list,
            "COLLECT_SET": exp.ArrayUniqueAgg.from_arg_list,
            "DATE_TRUNC": lambda args: exp.TimestampTrunc(
                this=seq_get(args, 0), unit=seq_get(args, 1)
            ),
            "DATE_ADD": exp.DateAdd.from_arg_list,
            "DATE_SUB": exp.DateSub.from_arg_list,
            "DATEDIFF": exp.DateDiff.from_arg_list,
            "FROM_UNIXTIME": exp.StrToUnix.from_arg_list,
            "GROUP_ARRAY": exp.ArrayAgg.from_arg_list,
            "GROUP_CONCAT": exp.GroupConcat.from_arg_list,
            "MONTHS_ADD": exp.AddMonths.from_arg_list,
            "NOW": exp.CurrentTimestamp.from_arg_list,
            "REGEXP": exp.RegexpLike.from_arg_list,
            "SIZE": exp.ArraySize.from_arg_list,
            "SPLIT_BY_STRING": exp.RegexpSplit.from_arg_list,
            "TO_DATE": exp.TsOrDsToDate.from_arg_list,
            "TRUNCATE": exp.Truncate.from_arg_list,
        }

        FUNCTION_PARSERS = {
            **MySQL.Parser.FUNCTION_PARSERS,
        }
        # Since it is incompatible with the implementation of mysql, we will pop it out here.
        FUNCTION_PARSERS.pop("GROUP_CONCAT")

        def _parse_explain(self) -> exp.Explain:
            this = "explain"
            comments = self._prev_comments
            return self.expression(
                exp.Explain,
                comments=comments,
                **{  # type: ignore
                    "this": this,
                    "expressions": self._parse_select(nested=True),
                },
            )

    class Tokenizer(MySQL.Tokenizer):
        KEYWORDS = {
            **MySQL.Tokenizer.KEYWORDS,
            "MATCH_ANY": TokenType.MATCH_ANY,
            "MATCH_ALL": TokenType.MATCH_ALL,
            "MATCH_PHRASE": TokenType.MATCH_PHRASE,
        }

    class Generator(MySQL.Generator):
        CAST_MAPPING = {}
        INTERVAL_ALLOWS_PLURAL_FORM = False
        LAST_DAY_SUPPORTS_DATE_PART = False

        STRING_TYPE_MAPPING = {
            exp.DataType.Type.TEXT: "STRING",
            exp.DataType.Type.TIME: "STRING",
            exp.DataType.Type.TINYTEXT: "STRING",
            exp.DataType.Type.MEDIUMTEXT: "STRING",
            exp.DataType.Type.LONGTEXT: "STRING",
            exp.DataType.Type.TINYBLOB: "STRING",
            exp.DataType.Type.MEDIUMBLOB: "STRING",
            exp.DataType.Type.LONGBLOB: "STRING",
            exp.DataType.Type.SET: "STRING",
            exp.DataType.Type.BINARY: "STRING",
            exp.DataType.Type.VARBINARY: "STRING",
            exp.DataType.Type.ENUM: "STRING",
            exp.DataType.Type.ENUM8: "STRING",
            exp.DataType.Type.ENUM16: "STRING",
            exp.DataType.Type.IPV4: "STRING",
            exp.DataType.Type.IPV6: "STRING",
            exp.DataType.Type.INT256: "STRING",
            exp.DataType.Type.UINT128: "STRING",
            exp.DataType.Type.UINT256: "STRING",
            exp.DataType.Type.TIMETZ: "STRING",
            exp.DataType.Type.INTERVAL: "STRING",
            exp.DataType.Type.UUID: "STRING",
            exp.DataType.Type.IPADDRESS: "STRING",
            exp.DataType.Type.GEOMETRY: "STRING",
        }

        TYPE_MAPPING = {
            **MySQL.Generator.TYPE_MAPPING,
            **STRING_TYPE_MAPPING,
            exp.DataType.Type.TIMESTAMP: "DATETIME",
            exp.DataType.Type.TIMESTAMPTZ: "DATETIME",
            exp.DataType.Type.BIT: "BOOLEAN",
            exp.DataType.Type.UTINYINT: "SMALLINT",
            exp.DataType.Type.USMALLINT: "INT",
            exp.DataType.Type.UMEDIUMINT: "INT",
            exp.DataType.Type.UINT: "BIGINT",
            exp.DataType.Type.UBIGINT: "LARGEINT",
            exp.DataType.Type.MEDIUMINT: "INT",
            exp.DataType.Type.YEAR: "SMALLINT",
            exp.DataType.Type.DATE32: "DATE",
            exp.DataType.Type.INT128: "LARGEINT",
            exp.DataType.Type.DATE: "DATE",
            exp.DataType.Type.UNKNOWN: "STRING",
            exp.DataType.Type.TINYINT: "TINYINT",
            exp.DataType.Type.SMALLINT: "SMALLINT",
            exp.DataType.Type.INT: "INT",
            exp.DataType.Type.BIGINT: "BIGINT",
            exp.DataType.Type.JSON: "JSON",
            exp.DataType.Type.BOOLEAN: "BOOLEAN",
            exp.DataType.Type.FLOAT: "FLOAT",
        }

        # Type correspondence between clickhouse and doris, used for STRUCT type adaptation
        CLICKHOUSE_TYPE_MAPPING = {
            "int8": "TINYINT",
            "int16": "SMALLINT",
            "int32": "INT",
            "int64": "BIGINT",
            "int128": "LARGEINT",
            "int256": "STRING",
            "uint8": "SMALLINT",
            "uint16": "INT",
            "uint32": "BIGINT",
            "uint64": "LARGEINT",
            "uint128": "STRING",
            "uint256": "STRING",
            "date32": "DATE",
            "datetime64": "DATETIME",
            "float32": "FLOAT",
            "float64": "DOUBLE",
            "enum": "STRING",
            "enum8": "STRING",
            "enum16": "STRING",
            "ipv4": "STRING",
            "ipv6": "STRING",
        }

        CLICKHOUSE_NOT_SUPPORT_TYPE = [
            exp.DataType.Type.AGGREGATEFUNCTION,
            exp.DataType.Type.SIMPLEAGGREGATEFUNCTION,
            exp.DataType.Type.LOWCARDINALITY,
            exp.DataType.Type.FIXEDSTRING,
        ]

        TIMESTAMP_FUNC_TYPES = set()

        PROPERTIES_LOCATION = {
            **MySQL.Generator.PROPERTIES_LOCATION,
            exp.PartitionedByProperty: exp.Properties.Location.UNSUPPORTED,
            exp.WithDataProperty: exp.Properties.Location.UNSUPPORTED,
            exp.EngineProperty: exp.Properties.Location.UNSUPPORTED,
            exp.AutoIncrementProperty: exp.Properties.Location.UNSUPPORTED,
            exp.CharacterSetProperty: exp.Properties.Location.UNSUPPORTED,
            exp.CollateProperty: exp.Properties.Location.UNSUPPORTED,
            exp.SchemaCommentProperty: exp.Properties.Location.UNSUPPORTED,
            exp.Order: exp.Properties.Location.UNSUPPORTED,
            exp.MergeTreeTTL: exp.Properties.Location.UNSUPPORTED,
            exp.SettingsProperty: exp.Properties.Location.UNSUPPORTED,
            exp.FileFormatProperty: exp.Properties.Location.UNSUPPORTED,
            exp.RowFormatDelimitedProperty: exp.Properties.Location.UNSUPPORTED,
            exp.ClusteredByProperty: exp.Properties.Location.UNSUPPORTED,
            exp.Property: exp.Properties.Location.UNSUPPORTED,
            exp.PrimaryKey: exp.Properties.Location.UNSUPPORTED,
            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
        }

        TRANSFORMS = {
            **MySQL.Generator.TRANSFORMS,
            exp.AddMonths: rename_func("MONTHS_ADD"),
            exp.AtTimeZone: attimezone_sql,
            exp.ApproxDistinct: approx_count_distinct_sql,
            exp.ApproxQuantile: rename_func("PERCENTILE_APPROX"),
            exp.ArgMax: rename_func("MAX_BY"),
            exp.ArgMin: rename_func("MIN_BY"),
            exp.ArrayAgg: rename_func("COLLECT_LIST"),
            exp.ArrayFilter: lambda self,
            e: f"ARRAY_FILTER({self.sql(e, 'expression')},{self.sql(e, 'this')})",
            exp.ArrayUniq: lambda self, e: f"SIZE(ARRAY_DISTINCT({self.sql(e, 'this')}))",
            exp.ArrayOverlaps: rename_func("ARRAYS_OVERLAP"),
            exp.ArrayPosition: rename_func("ARRAY_POSITION"),
            exp.ArrayElement: rename_func("ELEMENT_AT"),
            exp.ArrayStringConcat: handle_array_concat,
            exp.ArrayToString: handle_array_to_string,
            exp.ArrayUniqueAgg: rename_func("COLLECT_SET"),
            exp.AssertTrue: lambda self, e: self.func("IF", self.sql(e, "this"), "True", "Null"),
            exp.BitwiseAnd: rename_func("BITAND"),
            exp.BitwiseLeftShift: rename_func("BIT_SHIFT_LEFT"),
            exp.BitwiseRightShift: rename_func("BIT_SHIFT_RIGHT"),
            exp.BitwiseNot: rename_func("BITNOT"),
            exp.BitwiseOr: rename_func("BITOR"),
            exp.BitwiseXor: rename_func("BITXOR"),
            exp.BitmapXOrCount: rename_func("BITMAP_XOR_COUNT"),
            exp.CastToStrType: lambda self,
            e: f"CAST({self.sql(e, 'this')} AS {self.sql(e, 'to')})",
            exp.ConcatWs: handle_concat_ws,
            exp.CountIf: count_if_to_sum,
            exp.CurrentDate: no_paren_current_date_sql,
            exp.CurrentTimestamp: lambda *_: "NOW()",
            exp.DateDiff: handle_date_diff,
            exp.DPipe: lambda self, e: f"CONCAT({self.sql(e, 'this')},{self.sql(e, 'expression')})",
            exp.DateTrunc: handle_date_trunc,
            exp.Empty: rename_func("NULL_OR_EMPTY"),
            exp.Final: lambda self, e: f"{self.sql(e, 'this')}",
            exp.Filter: handle_filter,
            exp.GenerateSeries: handle_range,
            exp.GroupConcat: _string_agg_sql,
            exp.IsNotNull: rename_func("NOT_NULL_OR_EMPTY"),
            exp.JSONExtractScalar: _json_extract_sql,
            exp.JSONExtract: _json_extract_sql,
            exp.JSONBExtract: _json_extract_sql,
            exp.JSONBExtractScalar: _json_extract_sql,
            exp.JSONArrayContains: rename_func("JSON_CONTAINS"),
            exp.ParseJSON: rename_func("JSON_PARSE"),
            exp.JsonArrayLength: rename_func("JSON_LENGTH"),
            exp.Log: handle_log,
            exp.LTrim: rename_func("LTRIM"),
            exp.Map: rename_func("ARRAY_MAP"),
            exp.MonthsBetween: rename_func("MONTHS_DIFF"),
            exp.NotEmpty: rename_func("NOT_NULL_OR_EMPTY"),
            exp.QuartersAdd: lambda self,
            e: f"MONTHS_ADD({self.sql(e, 'this')},{3 * int(self.sql(e, 'expression'))})",
            exp.QuartersSub: lambda self,
            e: f"MONTHS_SUB({self.sql(e, 'this')},{3 * int(self.sql(e, 'expression'))})",
            exp.Rand: handle_rand,
            exp.RegexpLike: rename_func("REGEXP"),
            exp.RegexpExtract: handle_regexp_extract,
            exp.RegexpSplit: rename_func("SPLIT_BY_STRING"),
            exp.Replace: handle_replace,
            exp.RTrim: rename_func("RTRIM"),
            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
            exp.SHA2: lambda self, e: f"SHA2({self.sql(e, 'this')},{self.sql(e, 'length')})",
            exp.Shuffle: rename_func("ARRAY_SHUFFLE"),
            exp.Slice: rename_func("ARRAY_SLICE"),
            exp.SortArray: rename_func("ARRAY_SORT"),
            exp.Split: rename_func("SPLIT_BY_STRING"),
            exp.StAstext: handle_geography,
            exp.StrPosition: lambda self, e: (
                f"LOCATE({self.sql(e, 'substr')}, {self.sql(e, 'this')}, {self.sql(e, 'position')})"
                if self.sql(e, "position")
                else f"LOCATE({self.sql(e, 'substr')}, {self.sql(e, 'this')})"
            ),
            exp.StrToUnix: _str_to_unix_sql,
            exp.StrToDate: handle_to_date,
            exp.TimestampTrunc: timestamptrunc_sql,
            exp.TimeStrToDate: rename_func("TO_DATE"),
            exp.TimeStrToUnix: rename_func("UNIX_TIMESTAMP"),
            exp.TimeToUnix: rename_func("UNIX_TIMESTAMP"),
            exp.TimeToStr: handle_to_date,
            exp.Trim: _trim_sql,
            exp.ToChar: handle_to_char,
            exp.Today: lambda self, e: "TO_DATE(NOW())",
            exp.ToStartOfDay: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Day')",
            exp.ToStartOfHour: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Hour')",
            exp.ToStartOfMinute: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Minute')",
            exp.ToStartOfMonth: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Month')",
            exp.ToStartOfQuarter: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Quarter')",
            exp.ToStartOfSecond: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Second')",
            exp.ToStartOfWeek: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Week')",
            exp.ToStartOfYear: lambda self, e: f"DATE_TRUNC({self.sql(e, 'this')}, 'Year')",
            exp.ToYyyymm: lambda self, e: f"DATE_FORMAT({self.sql(e, 'this')}, '%Y%m')",
            exp.ToYyyymmdd: lambda self, e: f"DATE_FORMAT({self.sql(e, 'this')}, '%Y%m%d')",
            exp.ToYyyymmddhhmmss: lambda self,
            e: f"DATE_FORMAT({self.sql(e, 'this')}, '%Y%m%d%H%i%s')",
            exp.TsOrDsAdd: lambda self,
            e: f"DATE_ADD({self.sql(e, 'this')}, {self.sql(e, 'expression')})",  # Only for day level
            exp.TsOrDsToDate: handle_to_date,
            exp.UnixToStr: lambda self, e: self.func(
                "FROM_UNIXTIME", e.this, time_format("doris")(self, e)
            ),
            exp.UnixToTime: rename_func("FROM_UNIXTIME"),
            exp.Variance: rename_func("VAR_SAMP"),
        }

        def parameter_sql(self, expression: exp.Parameter) -> str:
            this = self.sql(expression, "this")
            expression_sql = self.sql(expression, "expression")

            parent = expression.parent
            this = f"{this}:{expression_sql}" if expression_sql else this

            if isinstance(parent, exp.EQ) and isinstance(parent.parent, exp.SetItem):
                # We need to produce SET key = value instead of SET ${key} = value
                return this

            return f"${{{this}}}"

        def explain_sql(self, expression: exp.Explain) -> str:
            this = self.sql(expression, "this")
            expr = self.sql(expression, "expressions")
            return f"{this} {expr}"

        def matchany_sql(self, expression: exp.MatchAny) -> str:
            return self.binary(expression, "MATCH_ANY")

        def matchall_sql(self, expression: exp.MatchAll) -> str:
            return self.binary(expression, "MATCH_ALL")

        def matchphrase_sql(self, expression: exp.MatchPhrase) -> str:
            return self.binary(expression, "MATCH_PHRASE")

        def in_sql(self, expression: exp.In) -> str:
            query = expression.args.get("query")
            unnest = expression.args.get("unnest")
            field = expression.args.get("field")
            # doris does not support global in
            # is_global = " GLOBAL" if expression.args.get("is_global") else ""

            if query:
                in_sql = self.wrap(self.sql(query))
            elif unnest:
                in_sql = self.in_unnest_op(unnest)
            elif field:
                in_sql = self.sql(field)
            else:
                in_sql = f"({self.expressions(expression, flat=True)})"

            return f"{self.sql(expression, 'this')} IN {in_sql}"

        def datatype_sql(self, expression: exp.DataType) -> str:
            root_expression: exp.Expression = expression
            while root_expression.parent is not None:
                root_expression = root_expression.parent

            # Only perform type conversion on create table statements
            if (
                not isinstance(root_expression, exp.Create)
                or not root_expression.args["kind"] == "TABLE"
            ):
                return generator.Generator.datatype_sql(self, expression)

            dialect = root_expression.args["dialect"]
            if dialect == "MYSQL":
                return self.mysql_type_to_doris(expression)
            elif dialect == "HIVE":
                return self.hive_type_to_doris(expression)
            elif dialect == "CLICKHOUSE":
                return self.clickhouse_type_to_doris(expression)
            elif dialect == "PRESTO":
                return self.presto_type_to_doris(expression)

            return generator.Generator.datatype_sql(self, expression)

        def create_sql(self, expression: exp.Create) -> str:
            # Only process create table
            if not expression.args["kind"] == "TABLE":
                return generator.Generator.create_sql(self, expression)

            # The 'create xxx like xxx' statement is not processed
            for e in expression.find_all(exp.LikeProperty):
                return generator.Generator.create_sql(self, expression)

            # todo The default value setting is not supported.
            for e in expression.this.expressions:
                if isinstance(e, exp.ColumnDef):
                    default = None
                    for constraint in e.constraints:
                        if isinstance(constraint.kind, exp.DefaultColumnConstraint):
                            default = constraint
                    if default:
                        e.constraints.remove(default)

            dialect = expression.args["dialect"]
            if dialect == "MYSQL":
                return self.create_mysql_sql(expression)
            elif dialect == "HIVE":
                return self.create_hive_sql(expression)
            elif dialect == "CLICKHOUSE":
                return self.create_clickhouse_sql(expression)
            elif dialect == "PRESTO":
                return self.create_presto_sql(expression)
            return generator.Generator.create_sql(self, expression)

        def createable_sql(self, expression: exp.Create, locations: t.DefaultDict) -> str:
            # Remove pk and index information and add them again after generating createable_sql to prevent subsequent create_sql() execution error
            remove_list = []
            expressions = expression.this.expressions
            for e in expressions:
                if isinstance(e, exp.PrimaryKey) or isinstance(e, exp.IndexColumnConstraint):
                    remove_list.append(e)

            for e in remove_list:
                expressions.remove(e)

            createable_sql = super().createable_sql(expression, locations)

            for e in remove_list:
                expressions.append(e)

            return createable_sql

        def mysql_type_to_doris(self, expression: exp.DataType) -> str:
            if expression.this in self.STRING_TYPE_MAPPING:
                return "STRING"
            elif expression.is_type(exp.DataType.Type.UDECIMAL):
                precision_expression = expression.find(exp.DataTypeParam)
                if precision_expression:
                    # If p + 1 > 38, type STRING will be used
                    precision = int(precision_expression.name) + 1
                    if precision <= 38:
                        precision_expression.this.set("this", precision)
                    else:
                        return "STRING"
            elif expression.is_type(exp.DataType.Type.CHAR):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "STRING" if size * 3 > 255 else f"CHAR({size * 3})"
            elif expression.is_type(exp.DataType.Type.VARCHAR):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "STRING" if size * 3 > 65533 else f"VARCHAR({size * 3})"
            elif expression.is_type(exp.DataType.Type.BIT):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "BOOLEAN" if size == 1 else "STRING"
            elif expression.this in (
                exp.DataType.Type.DATETIME,
                exp.DataType.Type.TIMESTAMP,
            ):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    precision = 6 if size > 6 else size
                    return f"DATETIME({precision})"

            return generator.Generator.datatype_sql(self, expression)

        def clickhouse_type_to_doris(self, expression: exp.DataType) -> str:
            if expression.this in self.CLICKHOUSE_NOT_SUPPORT_TYPE:
                raise TypeError(f"UNSUPPORTED TYPE: {expression.this.name}")

            if expression.this in self.STRING_TYPE_MAPPING:
                return "STRING"
            elif expression.this in (
                exp.DataType.Type.DATETIME,
                exp.DataType.Type.DATETIME64,
            ):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    precision = 6 if size > 6 else size
                    return f"DATETIME({precision})"
            elif expression.is_type(exp.DataType.Type.NULLABLE):
                # clickhouse has Nullable(xxx) type, doris takes xxx as the type
                expression = expression.expressions[0]
            elif expression.is_type(exp.DataType.Type.STRUCT):
                # The STRUCT type of clickhouse is STRUCT<String, String, Int>, while doris needs to be STRUCT<cnt_1:String,cnt_2:String,cnt_3:Int>, which needs to be converted.
                col_list = []
                for index, col in enumerate(expression.expressions, start=1):
                    col_type = col.this.lower()
                    col_list.append(
                        f"col_{index}: {self.CLICKHOUSE_TYPE_MAPPING.get(col_type, col_type).upper()}"
                    )
                cols = ", ".join(col_list)
                return f"STRUCT<{cols}>"

            return generator.Generator.datatype_sql(self, expression)

        def hive_type_to_doris(self, expression: exp.DataType) -> str:
            if expression.this in self.STRING_TYPE_MAPPING:
                return "STRING"
            elif expression.is_type(exp.DataType.Type.CHAR):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "STRING" if size * 3 > 255 else f"CHAR({size * 3})"
            elif expression.is_type(exp.DataType.Type.VARCHAR):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "STRING" if size * 3 > 65533 else f"VARCHAR({size * 3})"
            elif expression.this in (exp.DataType.Type.DATETIME, exp.DataType.Type.TIMESTAMP):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    precision = size if 0 <= size <= 6 else 0
                    return f"DATETIME({precision})"
            elif expression.is_type(expression.Type.STRUCT):
                # The STRUCT type of hive is STRUCT<tet1:STRING, test2:INT>, while the one read by doris is STRUCT<tet1 STRING, test2 INT>, which needs to be converted.
                col_list = []
                for col in expression.expressions:
                    st_type = self.datatype_sql(col.args["kind"])
                    col_list.append(f"{col.name}:{st_type}")
                cols = ", ".join(col_list)
                return f"STRUCT<{cols}>"

            return generator.Generator.datatype_sql(self, expression)

        def presto_type_to_doris(self, expression: exp.DataType) -> str:
            if expression.this in self.STRING_TYPE_MAPPING:
                return "STRING"
            elif expression.is_type(exp.DataType.Type.CHAR):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "STRING" if size * 3 > 255 else f"CHAR({size * 3})"
            elif expression.is_type(exp.DataType.Type.VARCHAR):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    return "STRING" if size * 3 > 65533 else f"VARCHAR({size * 3})"
            elif expression.this in (exp.DataType.Type.DATETIME, exp.DataType.Type.TIMESTAMP):
                size_expression = expression.find(exp.DataTypeParam)
                if size_expression:
                    size = int(size_expression.name)
                    precision = size if 0 <= size <= 6 else 0
                    return f"DATETIME({precision})"
            elif expression.is_type(expression.Type.STRUCT):
                # The STRUCT type of presto is STRUCT(tet1 STRING, test2 INT), while the type of doris is STRUCT<tet1:STRING, test2:INT>, which needs to be converted.
                col_list = []
                for col in expression.expressions:
                    st_type = self.datatype_sql(col.args["kind"])
                    col_list.append(f"{col.name}:{st_type}")
                cols = ", ".join(col_list)
                return f"STRUCT<{cols}>"

            return generator.Generator.datatype_sql(self, expression)

        def create_mysql_sql(self, expression: exp.Create) -> str:
            pk_list = []
            col_def_list = []
            for e in expression.this.expressions:
                if isinstance(e, exp.ColumnDef):
                    col_def_list.append(e)

            # Remove primary_key, auto_increment, and unique information
            for column in col_def_list:
                primary_key = None
                auto_increment = None
                unique = None
                for constraint in column.constraints:
                    if isinstance(constraint.kind, exp.PrimaryKeyColumnConstraint):
                        primary_key = constraint
                    if isinstance(constraint.kind, exp.AutoIncrementColumnConstraint):
                        auto_increment = constraint
                    if isinstance(constraint.kind, exp.UniqueColumnConstraint):
                        unique = constraint
                if primary_key and primary_key.parent:
                    pk_list.append(f"`{primary_key.parent.name}`")
                    if primary_key.parent.args["kind"].this.name == "TEXT":
                        primary_key.parent.set("kind", exp.DataType.build("varchar"))
                    column.constraints.remove(primary_key)
                if auto_increment:
                    column.constraints.remove(auto_increment)
                if unique:
                    column.constraints.remove(unique)

            first_data_type = expression.find(exp.DataType)
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.FLOAT,
                exp.DataType.Type.DOUBLE,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("decimal"))
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.TEXT,
                exp.DataType.Type.ARRAY,
                exp.DataType.Type.STRUCT,
                exp.DataType.Type.MAP,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("varchar"))

            if len(pk_list) == 0:
                for pk in expression.find_all(exp.PrimaryKey):
                    for e in pk:
                        pk_list.append(f"`{e.name}`")
                        for column in col_def_list:
                            if e.name == column.name and column.args["kind"].this.name == "TEXT":
                                column.set("kind", exp.DataType.build("varchar"))

            expression_sql = super().create_sql(expression)

            if pk_list:  # UNIQUE model
                pk_name = ", ".join(pk_list)
                return (
                    f"{expression_sql} "
                    f"UNIQUE KEY({pk_name}) "
                    f"DISTRIBUTED BY HASH({pk_name}) BUCKETS AUTO "
                    f"PROPERTIES ("
                    f'"replication_allocation" = "tag.location.default: 3"'
                    f")"
                )
            else:  # DUPLICATE model
                first_field_name = ""
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    first_field_name = col_def.name
                return (
                    f"{expression_sql} "
                    f"DUPLICATE KEY(`{first_field_name}`) "
                    f"DISTRIBUTED BY HASH(`{first_field_name}`) BUCKETS AUTO "
                    f"PROPERTIES ("
                    f'"replication_allocation" = "tag.location.default: 3"'
                    f")"
                )

        def create_clickhouse_sql(self, expression: exp.Create) -> str:
            first_data_type = expression.find(exp.DataType)
            # If is NULLABLE type, remove the NULLABLE keyword
            if first_data_type and first_data_type.is_type(exp.DataType.Type.NULLABLE):
                first_data_type = first_data_type.expressions[0]
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.FLOAT,
                exp.DataType.Type.DOUBLE,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("decimal"))
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.TEXT,
                exp.DataType.Type.ARRAY,
                exp.DataType.Type.STRUCT,
                exp.DataType.Type.MAP,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("varchar"))

            pk_list = []
            for pk in expression.find_all(exp.PrimaryKey):
                for e in pk:
                    pk_list.append(f"`{e.name}`")
                    for column in expression.this.expressions:
                        if (
                            isinstance(column, exp.ColumnDef)
                            and e.name == column.name
                            and column.args["kind"].this.name == "TEXT"
                        ):
                            column.set("kind", exp.DataType.build("varchar"))

            expression_sql = super().create_sql(expression)

            if pk_list:  # UNIQUE model
                pk_name = ", ".join(pk_list)
                return (
                    f"{expression_sql} "
                    f"UNIQUE KEY({pk_name}) "
                    f"DISTRIBUTED BY HASH({pk_name}) BUCKETS AUTO "
                    f"PROPERTIES ("
                    f'"replication_allocation" = "tag.location.default: 3"'
                    f")"
                )
            else:  # DUPLICATE model
                first_field_name = ""
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    first_field_name = col_def.name
                return (
                    f"{expression_sql} "
                    f"DUPLICATE KEY(`{first_field_name}`) "
                    f"DISTRIBUTED BY HASH(`{first_field_name}`) BUCKETS AUTO "
                    f"PROPERTIES ("
                    f'"replication_allocation" = "tag.location.default: 3"'
                    f")"
                )

        def create_hive_sql(self, expression: exp.Create) -> str:
            # Add the columns in the hive partition to the corresponding schema
            has_schema = isinstance(expression.this, exp.Schema)
            is_partition_table = expression.args.get("kind") in ("TABLE", "VIEW")
            parti_range_column = None
            disted_by_hash = None
            if has_schema and is_partition_table:
                # Look for PartitionedByProperty and ClusteredByProperty ahead of time
                partition_by_property = expression.find(exp.PartitionedByProperty)
                cluster_by_property = expression.find(exp.ClusteredByProperty)
                # If PartitionedByProperty is found, process the corresponding logic
                if partition_by_property is not None:
                    # Access directly instead of calling the find method again
                    for col in partition_by_property.this.expressions:
                        # Get the value of kind through args and query it once in TYPE_MAPPING
                        col_kind = col.args["kind"].this
                        col_type = self.TYPE_MAPPING.get(col_kind, col_kind)
                        # If the column type is DATE or DATETIME, set parti_range_column
                        if col_type in ("DATE", "DATETIME"):
                            # The auto partition column must be NOT NULL
                            constraints = col.args["constraints"]
                            notnull = exp.ColumnConstraint(kind=exp.NotNullColumnConstraint())
                            if notnull not in constraints:
                                constraints.insert(0, notnull)
                            parti_range_column = col.name
                        # Add column to expressions
                        expression.this.expressions.append(col)

                # If ClusteredByProperty is found, process the corresponding logic
                if cluster_by_property is not None:
                    clust_bukets = [
                        f"`{col_bukets.name}`" for col_bukets in cluster_by_property.expressions
                    ]
                    disted_by_hash = ", ".join(clust_bukets)

            # Determine the field type of the first column. If it does not comply with doris table building specifications, replace it with another data type.
            first_data_type = expression.find(exp.DataType)
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.FLOAT,
                exp.DataType.Type.DOUBLE,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("decimal"))
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.TEXT,
                exp.DataType.Type.ARRAY,
                exp.DataType.Type.STRUCT,
                exp.DataType.Type.MAP,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("varchar"))

            expression_sql = super().create_sql(expression)

            # Get the first column field
            first_field_name = ""
            col_def = expression.find(exp.ColumnDef)
            if col_def:
                first_field_name = col_def.name

            # Define the basic structure of SQL statements
            base_sql = f"{expression_sql} DUPLICATE KEY(`{first_field_name}`) "

            # DISTRIBUTED BY
            distributed_by_hash = (
                f"DISTRIBUTED BY HASH(`{first_field_name}`) BUCKETS AUTO"
                if disted_by_hash is None
                else f"DISTRIBUTED BY HASH({disted_by_hash}) BUCKETS AUTO"
            )

            # PARTITION BY
            auto_partition_by_range = ""
            dynamic_partition_properties = ""
            if parti_range_column is not None:
                auto_partition_by_range = (
                    f"AUTO PARTITION BY RANGE date_trunc(`{parti_range_column}`, 'month') () "
                )
                dynamic_partition_properties = (
                    '"dynamic_partition.enable" = "true", '
                    '"dynamic_partition.time_unit" = "month", '
                    '"dynamic_partition.prefix" = "p", '
                    '"dynamic_partition.end" = "3"'
                )

            # PROPERTIES
            common_properties = '"replication_allocation" = "tag.location.default: 3"'
            properties_part = f"{common_properties}"
            if dynamic_partition_properties:
                properties_part += f", {dynamic_partition_properties}"
            return (
                f"{base_sql}"
                f"{auto_partition_by_range}"
                f"{distributed_by_hash} "
                f"PROPERTIES ({properties_part})"
            )

            # todo bucket table、partition bucket table、ordinary table still need to be processed
            # if parti_range_column is not None and disted_by_hash is None:
            #     return (f"{expression_sql} "
            #             f"DUPLICATE KEY(`{first_field_name}`) "
            #             f"AUTO PARTITION BY RANGE date_trunc(`{parti_range_column}`, 'year') () "
            #             f"DISTRIBUTED BY HASH(`{first_field_name}`) BUCKETS AUTO "
            #             f"PROPERTIES ("
            #             f"\"replication_allocation\" = \"tag.location.default: 3\", "
            #             f"\"dynamic_partition.enable\" = \"true\", "
            #             f"\"dynamic_partition.time_unit\" = \"year\", "
            #             f"\"dynamic_partition.prefix\" = \"p\", "
            #             f"\"dynamic_partition.end\" = \"3\""
            #             f")")
            # # bucket table
            # elif disted_by_hash is not None and parti_range_column is None:
            #     return (f"{expression_sql} "
            #             f"DUPLICATE KEY(`{first_field_name}`) "
            #             f"DISTRIBUTED BY HASH({disted_by_hash}) BUCKETS AUTO "
            #             f"PROPERTIES ("
            #             f"\"replication_allocation\" = \"tag.location.default: 3\""
            #             f")")
            # # Partition bucket table
            # elif disted_by_hash is not None and parti_range_column is not None:
            #     return (f"{expression_sql} "
            #             f"DUPLICATE KEY(`{first_field_name}`) "
            #             f"AUTO PARTITION BY RANGE date_trunc(`{parti_range_column}`, 'year') () "
            #             f"DISTRIBUTED BY HASH({disted_by_hash}) BUCKETS AUTO "
            #             f"PROPERTIES ("
            #             f"\"replication_allocation\" = \"tag.location.default: 3\", "
            #             f"\"dynamic_partition.enable\" = \"true\", "
            #             f"\"dynamic_partition.time_unit\" = \"year\", "
            #             f"\"dynamic_partition.prefix\" = \"p\", "
            #             f"\"dynamic_partition.end\" = \"3\""
            #             f")")
            # # ordinary table
            # else:
            #     return (
            #         f"{expression_sql} "
            #         f"DUPLICATE KEY(`{first_field_name}`) "
            #         f"DISTRIBUTED BY HASH(`{first_field_name}`) BUCKETS AUTO "
            #         f"PROPERTIES ("
            #         f"\"replication_allocation\" = \"tag.location.default: 3\""
            #         f")"
            #     )

        def create_presto_sql(self, expression: exp.Create) -> str:
            # Obtain the corresponding partition and bucket columns according to the situation
            has_schema = isinstance(expression.this, exp.Schema)
            is_partition_table = expression.args.get("kind") in ("TABLE", "VIEW")
            parti_range_column = None
            disted_by_hash = None
            if has_schema and is_partition_table:
                # Look for PartitionedByProperty and ClusteredByProperty ahead of time
                partitioned_by_property_result = expression.find(exp.PartitionedByProperty)
                # If PartitionedByProperty is found, process the corresponding logic
                if partitioned_by_property_result is not None:
                    partitioned_expressions = partitioned_by_property_result.this.expressions
                    expression_expressions = expression.this.expressions
                    # For each partitioning column, check if there is a matching column of type 'DATE' or 'DATETIME'
                    for col in partitioned_expressions:
                        for col_test in expression_expressions:
                            col_this = col.this
                            # Confirm the type and check if the column name is within col_test.name
                            if col_this in col_test.name:
                                col_test_kind = col_test.args["kind"].this
                                col_test_type = self.TYPE_MAPPING.get(col_test_kind, col_test_kind)
                                if col_test_type in ("DATE", "DATETIME"):
                                    # The auto partition column must be NOT NULL
                                    constraints = col_test.args["constraints"]
                                    notnull = exp.ColumnConstraint(
                                        kind=exp.NotNullColumnConstraint()
                                    )
                                    if notnull not in constraints:
                                        constraints.insert(0, notnull)
                                    parti_range_column = col_this
                                    # After finding the column that meets the requirements, you can exit the inner loop
                                    break

                # Find the bucket column name based on the keyword 'bucketed_by'
                for porc in expression.find_all(exp.Property):
                    if porc.name.lower() == "bucketed_by":
                        clust_bukets = []
                        for bukec_inr in porc.args["value"].expressions:
                            formatted_name = "`%s`" % bukec_inr.this
                            clust_bukets.append(formatted_name)
                        disted_by_hash = ", ".join(clust_bukets)
            # Determine the field type of the first column. If it does not comply with doris table building specifications, replace it with another data type.
            first_data_type = expression.find(exp.DataType)
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.FLOAT,
                exp.DataType.Type.DOUBLE,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("decimal"))
            if first_data_type and first_data_type.this in (
                exp.DataType.Type.TEXT,
                exp.DataType.Type.ARRAY,
                exp.DataType.Type.STRUCT,
                exp.DataType.Type.MAP,
            ):
                col_def = expression.find(exp.ColumnDef)
                if col_def:
                    col_def.set("kind", exp.DataType.build("varchar"))

            expression_sql = super().create_sql(expression)

            # Get the first column field
            first_field_name = ""
            col_def = expression.find(exp.ColumnDef)
            if col_def:
                first_field_name = col_def.name

            # Define the basic structure of SQL statements
            base_sql = f"{expression_sql} DUPLICATE KEY(`{first_field_name}`) "

            # DISTRIBUTED BY
            distributed_by_hash = (
                f"DISTRIBUTED BY HASH(`{first_field_name}`) BUCKETS AUTO"
                if disted_by_hash is None
                else f"DISTRIBUTED BY HASH({disted_by_hash}) BUCKETS AUTO"
            )

            # PARTITION BY
            auto_partition_by_range = ""
            dynamic_partition_properties = ""
            if parti_range_column is not None:
                auto_partition_by_range = (
                    f"AUTO PARTITION BY RANGE date_trunc(`{parti_range_column}`, 'month') () "
                )
                dynamic_partition_properties = (
                    '"dynamic_partition.enable" = "true", '
                    '"dynamic_partition.time_unit" = "month", '
                    '"dynamic_partition.prefix" = "p", '
                    '"dynamic_partition.end" = "3"'
                )

            # PROPERTIES
            common_properties = '"replication_allocation" = "tag.location.default: 3"'
            properties_part = f"{common_properties}"
            if dynamic_partition_properties:
                properties_part += f", {dynamic_partition_properties}"
            return (
                f"{base_sql}"
                f"{auto_partition_by_range}"
                f"{distributed_by_hash} "
                f"PROPERTIES ({properties_part})"
            )
            # todo bucket table、partition bucket table、ordinary table still need to be processed

    KeyWords = [
        "ACCOUNT_LOCK",
        "ACCOUNT_UNLOCK",
        "ADD",
        "ADDDATE",
        "ADMIN",
        "AFTER",
        "AGG_STATE",
        "AGGREGATE",
        "ALIAS",
        "ALL",
        "ALTER",
        "ANALYZE",
        "ANALYZED",
        "AND",
        "ANTI",
        "APPEND",
        "ARRAY",
        "AS",
        "ASC",
        "AT",
        "AUTHORS",
        "AUTO",
        "AUTO_INCREMENT",
        "BACKEND",
        "BACKENDS",
        "BACKUP",
        "BEGIN",
        "BETWEEN",
        "BIGINT",
        "BIN",
        "BINARY",
        "BINLOG",
        "BITAND",
        "BITMAP",
        "BITMAP_UNION",
        "BITOR",
        "BITXOR",
        "BLOB",
        "BOOLEAN",
        "BRIEF",
        "BROKER",
        "BUCKETS",
        "BUILD",
        "BUILTIN",
        "BY",
        "CACHED",
        "CALL",
        "CANCEL",
        "CASE",
        "CAST",
        "CATALOG",
        "CATALOGS",
        "CHAIN",
        "CHAR",
        "CHARACTER",
        "CHARSET",
        "CHECK",
        "CLEAN",
        "CLUSTER",
        "CLUSTERS",
        "COLLATE",
        "COLLATION",
        "COLUMN",
        "COLUMNS",
        "COMMENT",
        "COMMIT",
        "COMMITTED",
        "COMPACT",
        "COMPLETE",
        "CONFIG",
        "CONNECTION",
        "CONNECTION_ID",
        "CONSISTENT",
        "CONSTRAINT",
        "CONSTRAINTS",
        "CONVERT",
        "COPY",
        "COUNT",
        "CREATE",
        "CREATION",
        "CRON",
        "CROSS",
        "CUBE",
        "CURRENT",
        "CURRENT_CATALOG",
        "CURRENT_DATE",
        "CURRENT_TIME",
        "CURRENT_TIMESTAMP",
        "CURRENT_USER",
        "DATA",
        "DATABASE",
        "DATABASES",
        "DATE",
        "DATE_ADD",
        "DATE_CEIL",
        "DATE_DIFF",
        "DATE_FLOOR",
        "DATE_SUB",
        "DATEADD",
        "DATEDIFF",
        "DATETIME",
        "DATETIMEV2",
        "DATEV2",
        "DATETIMEV1",
        "DATEV1",
        "DAY",
        "DAYS_ADD",
        "DAYS_SUB",
        "DECIMAL",
        "DECIMALV2",
        "DECIMALV3",
        "DECOMMISSION",
        "DEFAULT",
        "DEFERRED",
        "DELETE",
        "DEMAND",
        "DESC",
        "DESCRIBE",
        "DIAGNOSE",
        "DISK",
        "DISTINCT",
        "DISTINCTPC",
        "DISTINCTPCSA",
        "DISTRIBUTED",
        "DISTRIBUTION",
        "DIV",
        "DO",
        "DORIS_INTERNAL_TABLE_ID",
        "DOUBLE",
        "DROP",
        "DROPP",
        "DUPLICATE",
        "DYNAMIC",
        "ELSE",
        "ENABLE",
        "ENCRYPTKEY",
        "ENCRYPTKEYS",
        "END",
        "ENDS",
        "ENGINE",
        "ENGINES",
        "ENTER",
        "ERRORS",
        "EVENTS",
        "EVERY",
        "EXCEPT",
        "EXCLUDE",
        "EXECUTE",
        "EXISTS",
        "EXPIRED",
        "EXPLAIN",
        "EXPORT",
        "EXTENDED",
        "EXTERNAL",
        "EXTRACT",
        "FAILED_LOGIN_ATTEMPTS",
        "FALSE",
        "FAST",
        "FEATURE",
        "FIELDS",
        "FILE",
        "FILTER",
        "FIRST",
        "FLOAT",
        "FOLLOWER",
        "FOLLOWING",
        "FOR",
        "FOREIGN",
        "FORCE",
        "FORMAT",
        "FREE",
        "FROM",
        "FRONTEND",
        "FRONTENDS",
        "FULL",
        "FUNCTION",
        "FUNCTIONS",
        "GLOBAL",
        "GRANT",
        "GRANTS",
        "GRAPH",
        "GROUP",
        "GROUPING",
        "GROUPS",
        "HASH",
        "HAVING",
        "HDFS",
        "HELP",
        "HISTOGRAM",
        "HLL",
        "HLL_UNION",
        "HOSTNAME",
        "HOUR",
        "HUB",
        "IDENTIFIED",
        "IF",
        "IGNORE",
        "IMMEDIATE",
        "IN",
        "INCREMENTAL",
        "INDEX",
        "INDEXES",
        "INFILE",
        "INNER",
        "INSERT",
        "INSTALL",
        "INT",
        "INTEGER",
        "INTERMEDIATE",
        "INTERSECT",
        "INTERVAL",
        "INTO",
        "INVERTED",
        "IPV4",
        "IPV6",
        "IS",
        "IS_NOT_NULL_PRED",
        "IS_NULL_PRED",
        "ISNULL",
        "ISOLATION",
        "JOB",
        "JOBS",
        "JOIN",
        "JSON",
        "JSONB",
        "KEY",
        "KEYS",
        "KILL",
        "LABEL",
        "LARGEINT",
        "LAST",
        "LATERAL",
        "LDAP",
        "LDAP_ADMIN_PASSWORD",
        "LEFT",
        "LESS",
        "LEVEL",
        "LIKE",
        "LIMIT",
        "LINES",
        "LINK",
        "LIST",
        "LOAD",
        "LOCAL",
        "LOCALTIME",
        "LOCALTIMESTAMP",
        "LOCATION",
        "LOCK",
        "LOGICAL",
        "LOW_PRIORITY",
        "MANUAL",
        "MAP",
        "MATCH",
        "MATCH_ALL",
        "MATCH_ANY",
        "MATCH_ELEMENT_EQ",
        "MATCH_ELEMENT_GE",
        "MATCH_ELEMENT_GT",
        "MATCH_ELEMENT_LE",
        "MATCH_ELEMENT_LT",
        "MATCH_PHRASE",
        "MATCH_PHRASE_PREFIX",
        "MATCH_REGEXP",
        "MATERIALIZED",
        "MAX",
        "MAXVALUE",
        "MEMO",
        "MERGE",
        "MIGRATE",
        "MIGRATIONS",
        "MIN",
        "MINUS",
        "MINUTE",
        "MODIFY",
        "MONTH",
        "MTMV",
        "NAME",
        "NAMES",
        "NATURAL",
        "NEGATIVE",
        "NEVER",
        "NEXT",
        "NGRAM_BF",
        "NO",
        "NON_NULLABLE",
        "NOT",
        "NULL",
        "NULLS",
        "OBSERVER",
        "OF",
        "OFFSET",
        "ON",
        "ONLY",
        "OPEN",
        "OPTIMIZED",
        "OR",
        "ORDER",
        "OUTER",
        "OUTFILE",
        "OVER",
        "OVERWRITE",
        "PARAMETER",
        "PARSED",
        "PARTITION",
        "PARTITIONS",
        "PASSWORD",
        "PASSWORD_EXPIRE",
        "PASSWORD_HISTORY",
        "PASSWORD_LOCK_TIME",
        "PASSWORD_REUSE",
        "PATH",
        "PAUSE",
        "PERCENT",
        "PERIOD",
        "PERMISSIVE",
        "PHYSICAL",
        "PLAN",
        "PLUGIN",
        "PLUGINS",
        "POLICY",
        "PRECEDING",
        "PREPARE",
        "PRIMARY",
        "PROC",
        "PROCEDURE",
        "PROCESSLIST",
        "PROFILE",
        "PROPERTIES",
        "PROPERTY",
        "QUANTILE_STATE",
        "QUANTILE_UNION",
        "QUERY",
        "QUOTA",
        "RANDOM",
        "RANGE",
        "READ",
        "REAL",
        "REBALANCE",
        "RECOVER",
        "RECYCLE",
        "REFRESH",
        "REFERENCES",
        "REGEXP",
        "RELEASE",
        "RENAME",
        "REPAIR",
        "REPEATABLE",
        "REPLACE",
        "REPLACE_IF_NOT_NULL",
        "REPLICA",
        "REPOSITORIES",
        "REPOSITORY",
        "RESOURCE",
        "RESOURCES",
        "RESTORE",
        "RESTRICTIVE",
        "RESUME",
        "RETURNS",
        "REVOKE",
        "REWRITTEN",
        "RIGHT",
        "RLIKE",
        "ROLE",
        "ROLES",
        "ROLLBACK",
        "ROLLUP",
        "ROUTINE",
        "ROW",
        "ROWS",
        "S3",
        "SAMPLE",
        "SCHEDULE",
        "SCHEDULER",
        "SCHEMA",
        "SCHEMAS",
        "SECOND",
        "SELECT",
        "SEMI",
        "SERIALIZABLE",
        "SESSION",
        "SET",
        "SETS",
        "SHAPE",
        "SHOW",
        "SIGNED",
        "SKEW",
        "SMALLINT",
        "SNAPSHOT",
        "SONAME",
        "SPLIT",
        "SQL_BLOCK_RULE",
        "START",
        "STARTS",
        "STATS",
        "STATUS",
        "STOP",
        "STORAGE",
        "STREAM",
        "STREAMING",
        "STRING",
        "STRUCT",
        "SUBDATE",
        "SUM",
        "SUPERUSER",
        "SWITCH",
        "SYNC",
        "SYSTEM",
        "TABLE",
        "TABLES",
        "TABLESAMPLE",
        "TABLET",
        "TABLETS",
        "TASK",
        "TASKS",
        "TEMPORARY",
        "TERMINATED",
        "TEXT",
        "THAN",
        "THEN",
        "TIME",
        "TIMESTAMP",
        "TIMESTAMPADD",
        "TIMESTAMPDIFF",
        "TINYINT",
        "TO",
        "TRANSACTION",
        "TRASH",
        "TREE",
        "TRIGGERS",
        "TRIM",
        "TRUE",
        "TRUNCATE",
        "TYPE",
        "TYPECAST",
        "TYPES",
        "UNBOUNDED",
        "UNCOMMITTED",
        "UNINSTALL",
        "UNION",
        "UNIQUE",
        "UNLOCK",
        "UNSIGNED",
        "UPDATE",
        "USE",
        "USER",
        "USING",
        "VALUE",
        "VALUES",
        "VARCHAR",
        "VARIABLES",
        "VERBOSE",
        "VERSION",
        "VIEW",
        "WARNINGS",
        "WEEK",
        "WHEN",
        "WHERE",
        "WHITELIST",
        "WITH",
        "WORK",
        "WORKLOAD",
        "WRITE",
        "YEAR",
    ]
